"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.OutputPlugin = void 0;

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _webpack = _interopRequireDefault(require("webpack"));

var _env = require("../../env");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Plugin for copying generated files (bundle, chunks, assets) from Webpack's built location to the
 * React Native application directory, so that the files can be packed together into the `ipa`/`apk`.
 *
 * @category Webpack Plugin
 */
class OutputPlugin {
  /**
   * Constructs new `OutputPlugin`.
   *
   * @param config Plugin configuration options.
   */
  constructor(config) {
    this.config = config;
  }
  /**
   * Apply the plugin.
   *
   * @param compiler Webpack compiler instance.
   */


  apply(compiler) {
    var _process$env$CLI_OPTI;

    const cliOptions = JSON.parse((_process$env$CLI_OPTI = process.env[_env.CLI_OPTIONS_ENV_KEY]) !== null && _process$env$CLI_OPTI !== void 0 ? _process$env$CLI_OPTI : 'null'); // Noop when running from Webpack CLI or when running with dev server

    if (!cliOptions || 'start' in cliOptions.arguments || this.config.devServerEnabled) {
      return;
    }

    const logger = compiler.getInfrastructureLogger('ReactNativeOutputPlugin');
    const args = cliOptions.arguments.bundle;
    let {
      bundleOutput,
      assetsDest = '',
      sourcemapOutput = ''
    } = args;

    if (!_path.default.isAbsolute(bundleOutput)) {
      bundleOutput = _path.default.join(cliOptions.config.root, bundleOutput);
    }

    if (!sourcemapOutput) {
      sourcemapOutput = `${bundleOutput}.map`;
    }

    if (!_path.default.isAbsolute(sourcemapOutput)) {
      sourcemapOutput = _path.default.join(cliOptions.config.root, sourcemapOutput);
    }

    if (!assetsDest) {
      assetsDest = _path.default.dirname(bundleOutput);
    }

    let remoteChunksOutput = this.config.remoteChunksOutput;

    if (remoteChunksOutput && !_path.default.isAbsolute(remoteChunksOutput)) {
      remoteChunksOutput = _path.default.join(cliOptions.config.root, remoteChunksOutput);
    }

    logger.debug('Detected output paths:', {
      bundleOutput,
      sourcemapOutput,
      assetsDest,
      remoteChunksOutput
    });

    const isLocalChunk = chunkId => _webpack.default.ModuleFilenameHelpers.matchObject({
      include: this.config.localChunks
    }, chunkId);

    let entryGroup;
    const localChunks = [];
    const remoteChunks = [];
    compiler.hooks.compilation.tap('OutputPlugin', compilation => {
      compilation.hooks.afterProcessAssets.tap('OutputPlugin', assets => {
        entryGroup = compilation.chunkGroups.find(group => group.isInitial());
        const sharedChunks = new Set();
        let entryChunk;

        for (const chunk of compilation.chunks) {
          var _entryGroup, _chunk$name, _chunk$id;

          // Do not process shared chunks right now.
          if (sharedChunks.has(chunk)) {
            continue;
          }

          [...chunk.getAllInitialChunks()].filter(sharedChunk => sharedChunk !== chunk).forEach(sharedChunk => {
            sharedChunks.add(sharedChunk);
          }); // Entry chunk

          if (((_entryGroup = entryGroup) === null || _entryGroup === void 0 ? void 0 : _entryGroup.chunks[0]) === chunk) {
            entryChunk = chunk;
            localChunks.push(chunk);
          } else if (isLocalChunk((_chunk$name = chunk.name) !== null && _chunk$name !== void 0 ? _chunk$name : (_chunk$id = chunk.id) === null || _chunk$id === void 0 ? void 0 : _chunk$id.toString())) {
            localChunks.push(chunk);
          } else {
            remoteChunks.push(chunk);
          }
        } // Process shared chunks to add them either as local or remote chunk.


        for (const sharedChunk of sharedChunks) {
          var _sharedChunk$name, _sharedChunk$id;

          const isUsedByLocalChunk = localChunks.some(localChunk => {
            return [...localChunk.getAllInitialChunks()].includes(sharedChunk);
          });

          if (isUsedByLocalChunk || isLocalChunk((_sharedChunk$name = sharedChunk.name) !== null && _sharedChunk$name !== void 0 ? _sharedChunk$name : (_sharedChunk$id = sharedChunk.id) === null || _sharedChunk$id === void 0 ? void 0 : _sharedChunk$id.toString())) {
            localChunks.push(sharedChunk);
          } else {
            remoteChunks.push(sharedChunk);
          }
        }

        if (!entryChunk) {
          throw new Error('Cannot infer entry chunk - this should have not happened.');
        }

        const mainBundleAssetName = [...entryChunk.files][0];
        const mainBundleSource = assets[mainBundleAssetName];
        assets[mainBundleAssetName] = new _webpack.default.sources.ConcatSource(`var __CHUNKS__=${JSON.stringify({
          local: localChunks.map(localChunk => {
            var _localChunk$name;

            return (_localChunk$name = localChunk.name) !== null && _localChunk$name !== void 0 ? _localChunk$name : localChunk.id;
          })
        })};\n`, mainBundleSource);
      });
    });
    compiler.hooks.afterEmit.tapPromise('OutputPlugin', async compilation => {
      const outputPath = compilation.outputOptions.path;

      if (!outputPath) {
        throw new Error('Cannot infer output path from compilation');
      }

      const promises = [];

      for (const chunk of localChunks) {
        var _entryGroup2;

        // Process entry chunk
        if (((_entryGroup2 = entryGroup) === null || _entryGroup2 === void 0 ? void 0 : _entryGroup2.chunks[0]) === chunk) {
          promises.push(...this.processEntryChunkFiles(compilation, chunk, outputPath, bundleOutput, sourcemapOutput, logger));
        } else {
          promises.push(...this.processChunkFiles(compilation, chunk, outputPath, assetsDest, logger));
        }

        promises.push(...this.processChunkAuxiliaryFiles(chunk, outputPath, assetsDest, logger));
      }

      if (remoteChunksOutput) {
        for (const chunk of remoteChunks) {
          promises.push(...this.processChunkFiles(compilation, chunk, outputPath, remoteChunksOutput, logger));
          promises.push(...this.processChunkAuxiliaryFiles(chunk, outputPath, remoteChunksOutput, logger));
        }
      }

      await Promise.all(promises);
    });
  }

  async copyAsset(from, to, logger) {
    logger.debug('Copying asset:', from, 'to:', to);
    await _fsExtra.default.ensureDir(_path.default.dirname(to));
    await _fsExtra.default.copyFile(from, to);
  }

  processEntryChunkFiles(compilation, chunk, outputPath, bundleOutput, sourcemapOutput, logger) {
    var _compilation$assetsIn, _compilation$assetsIn2;

    const promises = [];
    const [bundleFile] = [...chunk.files];
    const relatedSourceMap = (_compilation$assetsIn = compilation.assetsInfo.get(bundleFile)) === null || _compilation$assetsIn === void 0 ? void 0 : (_compilation$assetsIn2 = _compilation$assetsIn.related) === null || _compilation$assetsIn2 === void 0 ? void 0 : _compilation$assetsIn2.sourceMap;
    const sourceMapFile = Array.isArray(relatedSourceMap) ? relatedSourceMap[0] : relatedSourceMap;
    promises.push(this.copyAsset(_path.default.join(outputPath, bundleFile), bundleOutput, logger));

    if (sourceMapFile) {
      promises.push(this.copyAsset(_path.default.join(outputPath, sourceMapFile), sourcemapOutput, logger));
    }

    return promises;
  }

  processChunkFiles(compilation, chunk, outputPath, assetsDest, logger) {
    var _compilation$assetsIn3, _compilation$assetsIn4;

    const promises = [];
    const [chunkFile] = [...chunk.files];
    const relatedSourceMap = (_compilation$assetsIn3 = compilation.assetsInfo.get(chunkFile)) === null || _compilation$assetsIn3 === void 0 ? void 0 : (_compilation$assetsIn4 = _compilation$assetsIn3.related) === null || _compilation$assetsIn4 === void 0 ? void 0 : _compilation$assetsIn4.sourceMap;
    const sourceMapFile = Array.isArray(relatedSourceMap) ? relatedSourceMap[0] : relatedSourceMap;
    promises.push(this.copyAsset(_path.default.join(outputPath, chunkFile), _path.default.join(assetsDest, chunkFile), logger));

    if (sourceMapFile) {
      promises.push(this.copyAsset(_path.default.join(outputPath, sourceMapFile), _path.default.join(assetsDest, sourceMapFile), logger));
    }

    return promises;
  }

  processChunkAuxiliaryFiles(chunk, outputPath, assetsDest, logger) {
    const assets = [...chunk.auxiliaryFiles].filter(file => !/\.map$/.test(file));
    return assets.map(asset => this.copyAsset(_path.default.join(outputPath, asset), _path.default.join(assetsDest, asset), logger));
  }

}

exports.OutputPlugin = OutputPlugin;
//# sourceMappingURL=OutputPlugin.js.map